@@ -1,6 +1,6 @@
 /*
   zip_unchange.c -- undo changes to file in zip archive
-  Copyright (C) 1999-2007 Dieter Baron and Thomas Klausner
+  Copyright (C) 1999-2014 Dieter Baron and Thomas Klausner
 
   This file is part of libzip, a library to manipulate ZIP archives.
   The authors can be contacted at <libzip@nih.at>
@@ -31,53 +31,39 @@
   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-
 
 #include <stdlib.h>
 
 #include "zipint.h"
 
-
 
 ZIP_EXTERN int
-zip_unchange(struct zip *za, zip_uint64_t idx)
+zip_unchange(zip_t *za, zip_uint64_t idx)
 {
     return _zip_unchange(za, idx, 0);
 }
 
-
 
 int
-_zip_unchange(struct zip *za, zip_uint64_t idx, int allow_duplicates)
+_zip_unchange(zip_t *za, zip_uint64_t idx, int allow_duplicates)
 {
-    int i;
+    zip_int64_t i;
     
     if (idx >= za->nentry) {
-	_zip_error_set(&za->error, ZIP_ER_INVAL, 0);
+	zip_error_set(&za->error, ZIP_ER_INVAL, 0);
 	return -1;
     }
 
-    if (za->entry[idx].ch_filename) {
-	if (!allow_duplicates) {
-	    i = _zip_name_locate(za,
-			 _zip_get_name(za, idx, ZIP_FL_UNCHANGED, NULL),
-				 0, NULL);
-	    if (i != -1 && i != idx) {
-		_zip_error_set(&za->error, ZIP_ER_EXISTS, 0);
-		return -1;
-	    }
+    if (!allow_duplicates && za->entry[idx].changes && (za->entry[idx].changes->changed & ZIP_DIRENT_FILENAME)) {
+	i = _zip_name_locate(za, _zip_get_name(za, idx, ZIP_FL_UNCHANGED, NULL), 0, NULL);
+	if (i >= 0 && (zip_uint64_t)i != idx) {
+	    zip_error_set(&za->error, ZIP_ER_EXISTS, 0);
+	    return -1;
 	}
-
-	free(za->entry[idx].ch_filename);
-	za->entry[idx].ch_filename = NULL;
     }
 
-    free(za->entry[idx].ch_extra);
-    za->entry[idx].ch_extra = NULL;
-    za->entry[idx].ch_extra_len = -1;
-    free(za->entry[idx].ch_comment);
-    za->entry[idx].ch_comment = NULL;
-    za->entry[idx].ch_comment_len = -1;
+    _zip_dirent_free(za->entry[idx].changes);
+    za->entry[idx].changes = NULL;
 
     _zip_unchange_data(za->entry+idx);
 
